<script setup lang="ts">

import dayjs from 'dayjs'
import {ScreenApi, statusMap, CallingVO} from "@/api/ecg/screen";

defineOptions({ name: 'bigscreen' })

const listReady = ref<CallingVO[]>([]) // 列表的数据
const listWaiting = ref<CallingVO[]>([]) // 列表的数据
const listPassed = ref<CallingVO[]>([]) //

let curSpeakPat : CallingVO | undefined = undefined;
let curSpeakSeqNum : number = 0
let curWorkDay : Date | undefined = undefined

const getList = async () => {
  const data = await ScreenApi.getBigScreenData()
  listReady.value = data[1]
  listWaiting.value = data[2]
  listPassed.value = data[3]
}

const startScrolling = () => {
  setInterval(() => {
    getList();
    if ( curSpeakPat === undefined ) {
      initiateSpeak()
    }
  }, 5000); // 每X秒滚动一次
}

onMounted( () => {
  startScrolling()
})

const onSpeachEndEvent = async (event) => {
  console.log("Speech ended... " + event.currentTarget.text);
  await ScreenApi.markCalled(curSpeakPat!.patId)
  initiateSpeak()
}

const getNextSpeak = () => {
  if (listReady.value.length === 0) {
    return undefined;
  }

  // 处理每天 反转的情况
  if ( curWorkDay === undefined) {
    curSpeakSeqNum = 0;
    curWorkDay = new Date()
  } else if (!isSameDate(dayjs(listReady.value[0].bookDate).toDate(), curWorkDay)) {
    curSpeakSeqNum = 0;
    curWorkDay = dayjs(listReady.value[0].bookDate).toDate()
  }

  return listReady.value.find( item => item.status === 30 && item.called === 0 && item.seqNum > curSpeakSeqNum)
}

const initiateSpeak = () => {
  const queuePat = getNextSpeak()
  if (queuePat !== undefined) {
    curSpeakPat = queuePat
    curSpeakSeqNum = curSpeakPat.seqNum
    speak("请、" + queuePat.patName + "到" + queuePat.roomName + "就诊");
  } else {
    curSpeakPat = undefined
  }
}

const speak = (msg) => {
  console.info("speak " + msg);
  var speech = new SpeechSynthesisUtterance()
  speech.text = msg + "。。。" + msg + "。。。" + msg  + "。。。"
  speech.pitch = 1 // 获取并设置话语的音调(0-2 默认1，值越大越尖锐,越低越低沉)
  speech.rate = 0.9 // 获取并设置说话的速度(0.1-10 默认1，值越大语速越快,越小语速越慢)
  speech.volume = 100 // 获取并设置说话的音量
  speech.lang = 'zh-CN' // 设置播放语言

  speech.onend = onSpeachEndEvent
  speechSynthesis.speak(speech)
}

const isSameDate = ( date1: Date, date2 : Date) => {
  if (date1.getFullYear() === date2.getFullYear()
      && date1.getMonth() === date2.getMonth()
      && date1.getDate() === date2.getDate())
    return true
  else
    return false
}

</script>

<template>
  <el-container style="height: 100%;">
    <el-header>
      <img
          height="40px"
          src="@/assets/imgs/jinhua.png"
      />
      金华人民医院
    </el-header>
    <el-container>
      <el-aside width="500px">
        <el-table
            :data="listReady"
            stripe
            :show-header="false"
            style="width: 100%; height: 100%; border: solid var(--el-color-primary-light-7); font-size: 18px;">
          <el-table-column
              prop="patName"
              label="患者姓名"
              width="80"/>
          <el-table-column label="状态" align="center" prop="status" width="80">
            <template #default="scope">
              {{ statusMap.get(scope.row.status) }}
            </template>
          </el-table-column>
          <el-table-column
              prop="roomName"
              label="诊室"
              width="120"/>
        </el-table>
      </el-aside>
      <el-container>
        <el-main style="padding: 0 0 ">
          <el-header height="30px" style="background-color: #98b8e5; line-height: 30px;">等候区</el-header>
          <span v-for="(waitingItem, index) in listWaiting" :key="index">
            {{waitingItem.patName + "&nbsp;&nbsp;&nbsp;&nbsp;"}}
          </span>
        </el-main>
        <el-footer height="100px" style="padding: 0 0">
          <el-header height="30px" style="background-color: #98b8e5; line-height: 30px;">过号区</el-header>
          <span v-for="(passedItem, index) in listPassed" :key="index">
            {{passedItem.patName + "&nbsp;&nbsp;&nbsp;&nbsp;"}}
          </span>
        </el-footer>
      </el-container>
    </el-container>
    <el-button @click="speak('欢迎使用')" >欢迎使用</el-button>
  </el-container>
</template>

<style scoped lang="scss">
.el-header {
  background-color: var(--el-color-primary-light-7);
  color: #333;
  text-align: center;
  line-height: 40px;

  display: flex;
  align-items: center;
  justify-content: center;
}

.el-footer {
  //background-color: var(--el-color-primary-light-7);
  color: #333;
  text-align: center;
  line-height: 60px;
}

.el-aside {
  background-color: var(--el-color-primary-light-7);
  color: #333;
  text-align: center;
  line-height: 200px;
}

.el-main {
  //background-color: var(--el-color-primary-light-7);
  color: #333;
  text-align: center;
  line-height: 160px;
}

body > .el-container {
  margin-bottom: 40px;
}

</style>
